@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  * Copyright 2011 Action Without Borders
  *
@@ -249,55 +249,7 @@ DEFINE_TESTCASE(wildquery1, backend) {
     return true;
 }
 
-// tests a query across multiple databases
-DEFINE_TESTCASE(multidb1, backend) {
-    Xapian::Database mydb1(get_database("apitest_simpledata", "apitest_simpledata2"));
-    Xapian::Enquire enquire1(mydb1);
-
-    Xapian::Database mydb2(get_database("apitest_simpledata"));
-    mydb2.add_database(get_database("apitest_simpledata2"));
-    Xapian::Enquire enquire2(mydb2);
-
-    // make a simple query, with one word in it - "word".
-    Xapian::Query myquery("word");
-    enquire1.set_query(myquery);
-    enquire2.set_query(myquery);
-
-    // retrieve the top ten results from each method of accessing
-    // multiple text files
-    Xapian::MSet mymset1 = enquire1.get_mset(0, 10);
-    Xapian::MSet mymset2 = enquire2.get_mset(0, 10);
-
-    TEST_EQUAL(mymset1.size(), mymset2.size());
-    TEST(mset_range_is_same_weights(mymset1, 0, mymset2, 0, mymset1.size()));
-    return true;
-}
-
-// tests a query across multiple databases with terms only
-// in one of the two databases
-DEFINE_TESTCASE(multidb2, backend && !multi) {
-    Xapian::Database mydb1(get_database("apitest_simpledata",
-				  "apitest_simpledata2"));
-    Xapian::Enquire enquire1(mydb1);
-
-    Xapian::Database mydb2(get_database("apitest_simpledata"));
-    mydb2.add_database(get_database("apitest_simpledata2"));
-    Xapian::Enquire enquire2(mydb2);
-
-    // make a simple query
-    Xapian::Query myquery = query(Xapian::Query::OP_OR, "inmemory", "word");
-    enquire1.set_query(myquery);
-    enquire2.set_query(myquery);
-
-    // retrieve the top ten results from each method of accessing
-    // multiple text files
-    Xapian::MSet mymset1 = enquire1.get_mset(0, 10);
-    Xapian::MSet mymset2 = enquire2.get_mset(0, 10);
-
-    TEST_EQUAL(mymset1.size(), mymset2.size());
-    TEST(mset_range_is_same_weights(mymset1, 0, mymset2, 0, mymset1.size()));
-    return true;
-}
+// multidb1 and multidb2 no longer exist.
 
 // test that a multidb with 2 dbs query returns correct docids
 DEFINE_TESTCASE(multidb3, backend && !multi) {
@@ -920,11 +872,13 @@ DEFINE_TESTCASE(reversebool1, backend) {
     {
 	Xapian::MSetIterator i = mymset1.begin();
 	Xapian::MSetIterator j = mymset2.begin();
-	for ( ; i != mymset1.end(), j != mymset2.end(); ++i, j++) {
+	for ( ; i != mymset1.end(); ++i, j++) {
+	    TEST(j != mymset2.end());
 	    // if this fails, then setting match_sort_forward=true was not
 	    // the same as the default.
 	    TEST_EQUAL(*i, *j);
 	}
+	TEST(j == mymset2.end());
     }
 
     // mymset1 and mymset3 should be same but reversed
@@ -932,13 +886,11 @@ DEFINE_TESTCASE(reversebool1, backend) {
 
     {
 	Xapian::MSetIterator i = mymset1.begin();
-	vector<Xapian::docid> rev(mymset3.begin(), mymset3.end());
-	// Next iterator not const because of compiler brokenness (egcs 1.1.2)
-	vector<Xapian::docid>::reverse_iterator j = rev.rbegin();
-	for ( ; i != mymset1.end(); ++i, j++) {
+	Xapian::MSetIterator j = mymset3.end();
+	for ( ; i != mymset1.end(); ++i) {
 	    // if this fails, then setting match_sort_forward=false didn't
 	    // reverse the results.
-	    TEST_EQUAL(*i, *j);
+	    TEST_EQUAL(*i, *--j);
 	}
     }
 
@@ -967,24 +919,25 @@ DEFINE_TESTCASE(reversebool2, backend) {
     {
 	Xapian::MSetIterator i = mymset1.begin();
 	Xapian::MSetIterator j = mymset2.begin();
-	for ( ; i != mymset1.end(), j != mymset2.end(); ++i, j++) {
+	for ( ; j != mymset2.end(); ++i, j++) {
+	    TEST(i != mymset1.end());
 	    // if this fails, then setting match_sort_forward=true was not
 	    // the same as the default.
 	    TEST_EQUAL(*i, *j);
 	}
+	// mymset1 should be larger.
+	TEST(i != mymset1.end());
     }
 
     // mymset3 should be last msize items of mymset1, in reverse order
     TEST_EQUAL(msize, mymset3.size());
     {
-	vector<Xapian::docid> rev(mymset1.begin(), mymset1.end());
-	// Next iterator not const because of compiler brokenness (egcs 1.1.2)
-	vector<Xapian::docid>::reverse_iterator i = rev.rbegin();
-	Xapian::MSetIterator j = mymset3.begin();
-	for ( ; j != mymset3.end(); ++i, j++) {
+	Xapian::MSetIterator i = mymset1.end();
+	Xapian::MSetIterator j;
+	for (j = mymset3.begin(); j != mymset3.end(); j++) {
 	    // if this fails, then setting match_sort_forward=false didn't
 	    // reverse the results.
-	    TEST_EQUAL(*i, *j);
+	    TEST_EQUAL(*--i, *j);
 	}
     }
 
@@ -2303,13 +2256,17 @@ DEFINE_TESTCASE(tradweight1, backend) {
     enquire.set_query(Xapian::Query("word") );
 
     Xapian::MSet mset = enquire.get_mset(0, 25);
+    TEST_EQUAL(mset.size(), 2);
 
     enquire.set_weighting_scheme(Xapian::TradWeight(0));
-    enquire.set_query(Xapian::Query("word") );
+    enquire.set_query(Xapian::Query("this") );
 
     mset = enquire.get_mset(0, 25);
-    // FIXME: should check that TradWeight(0) means wdf and doc length really
-    // don't affect the weights as stated in the documentation.
+    TEST_EQUAL(mset.size(), 6);
+
+    // Check that TradWeight(0) means wdf and doc length really don't affect
+    // the weights as stated in the documentation.
+    TEST_EQUAL(mset[0].get_weight(), mset[5].get_weight());
 
     return true;
 }
